#!/usr/bin/env python
#
# Copy the parameters from Velocity and use the Redshift to calculate the cross-correlation between
# the template and the object.


#
#  (1) emask has been split into emask + efind
#      emask: controls the clipping of emission lines out of the spectra
#             before cross-correlation [now set emask=0 when efit=1]
#      efind: controls the *search* for emission lines after the cross-
#             correlation is performed [now set efind to threshold, 3?]
#  (2) esig:  this is "trlim" internally controls something i can't
#             quite remember. has to do with an integrated significance
#             of a feature. set to something same number as efind.
#  (3) efilt: this is the scale over which the spectra are filtered in
#             order to subtract off continuum to then grab emission
#             features. set to something like 10.
#               
#  IOW:
#     emask = 0   -- always
#     efind = 0   -- for boring galaxies and broad-line AGN (i.e. QSOs)
#     efind != 0  -- for narrow emission line objects
#     esig  != 0  -- for narrow emission line objects
#     efilt != 0  -- for narrow emission line objects
#
#  IOW:
#     for all objects, CC gets done.
#     if efind>0: subsequent search for emission features is done
#
#


from Redshifts import *
import time,shutil,os

# Temp aRedshift.py Makefile
# Need to add arguments for makefile.
def autoRedMake(fits):
    makefile = "Makefile"
    images = open(fits,"r").readlines()
    f = open(makefile,"w")
    f.write("# Redshift Makefile\n\n")
    f.write("stage-redshift:\n")
    for image in images:
        f.write("\taRedshift.py %s -nz 100\n" % (image.strip()))
    f.write("\ttouch stage-redshift\n")
    f.close()
    os.chmod(makefile,493)


### Hard-wired parameters
ph1 = os.getenv("PYTHONHOME1")
if not ph1: raise "PYTHONHOME1 environmental variable not set!"

TemplatePath = "/".join([ph1,"/templates"])

CnWaveDict = {2.037:6600,1.341:6700,0.583:7700}

RedshiftBins = {}
RedshiftBins["galaxy"] = [(0.1,0.2,0),(0.35,0.4,0),(0.55,0.4,0),(0.80,0.4,0),(1.5,1.2,0)]
RedshiftBins["agn"] = [(0.75,0.5,0),(1.5,1.2,0),(3.0,2.0,0),(5.0,2.5,0)]
RedshiftBins["star"] = [(0.0,0.05,0)]

FilterParams = {}
#FilterParams["galaxy"] = (0,0,4)
#FilterParams["agn"] = (0,0,4)
#FilterParams["star"] = (0,0,6)
FilterParams["galaxy"] = (30,0,0,0)
FilterParams["agn"] = (0,0,0,4)
FilterParams["star"] = (0,0,0,6)

TemplateIDs = {}
TemplateIDs["galaxy"] = [23,24,25,26,27,28]
TemplateIDs["agn"] = [29,30,31,32]
TemplateIDs["star"] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22]

### Command-line arguements
argv,agn=rdarg(argv,"-agn",None,0,single=1)
argv,star=rdarg(argv,"-star",None,0,single=1)
argv,galaxy=rdarg(argv,"-galaxy",None,0,single=1)
argv,makefile=rdarg(argv,"-mk",None,0,single=1)
argv,fits=rdarg(argv,"-fits",None,None)
argv,sloan=rdarg(argv,"-sloan",None,0,single=1)
argv,imacs=rdarg(argv,"-imacs",None,0,single=1)
argv,input=rdarg(argv,"-input",None,None)
argv,sngle=rdarg(argv,"-single",None,0,single=1)
argv,direct = rdarg(argv,"-d",None,"redshiftCC")

### Emission line binning
argv,ebin= rdarg(argv,"-ebin",None,0,single=1)

if makefile: autoRedMake(fits); sys.exit()
argv,inctitle=rdarg(argv,"-notitle",None,1,single=1)
argv,rescale=rdarg(argv,"-nochi2",None,1,single=1)
argv,efit=rdarg(argv,"-efit",None,0,single=1)
argv,watch=rdarg(argv,"-watch",None,0,single=1)
argv,inter = rdarg(argv,"-inter",None,0,single=1)
argv,pgdev = rdarg(argv,"-d",None,getpgdev()[0])
if watch: watchdev = pgdev
else: watchdev = "/null"
argv,pgsize = rdarg(argv,"-s",float,getpgdev()[1])
argv,template = rdarg(argv,"-t",None,None)
argv,trow= rdarg(argv,"-tr",int,0)
argv,tk= rdarg(argv,"-tk",int,3)
argv,grow= rdarg(argv,"-gr",int,0)
argv,nrow= rdarg(argv,"-nr",int,1)
argv,w1= rdarg(argv,"-w1",float,0)
argv,w2= rdarg(argv,"-w2",float,0)
argv,adu = rdarg(argv,"-g",float,2)
argv,rn = rdarg(argv,"-rn",float,6)
argv,bfilt = rdarg(argv,"-bf",float,0)
argv,ffilt = rdarg(argv,"-ff",int,0)
argv,pfilt = rdarg(argv,"-pf",int,0)
argv,verb= rdarg(argv,"-v",None,0,single=1)
argv,dps= rdarg(argv,"-dps",None,0,single=1)
argv,tps = rdarg(argv,"-tps",None,0,single=1)
argv,prompt= rdarg(argv,"-prompt",None,0,single=1)
argv,dump= rdarg(argv,"-dump",None,0,single=1)
argv,amask= rdarg(argv,"-atmos",None,1,single=1)
argv,uniform= rdarg(argv,"-uniform",None,0,single=1)
argv,bmask= rdarg(argv,"-balmer",None,0,single=1)
argv,emask= rdarg(argv,"-emask",float,0.0)
argv,efind= rdarg(argv,"-efind",float,[0.0,3.0][efit])
argv,efilt= rdarg(argv,"-efilt",float,[0.0,15.0][efit])
argv,trlim= rdarg(argv,"-esig",float,[0.0,2.0][efit])
argv,citer= rdarg(argv,"-iter",int,4)
argv,nz= rdarg(argv,"-nz",int,0)
argv,pu= rdarg(argv,"-p",float,0)

inspec  = argv[1]

if not os.path.exists(direct): os.mkdir(direct)


if dps or tps: pgdev = replace(inspec,".fits","_cc.ps/vcps")

f = FITS(inspec)
naxis3 = f["naxis3"]
if sloan:
    gspec = f.row(grow)
    if nrow:
      nspec = f.row(nrow)
      estnoise = 0
    else:
      try: nex = float(f["nextract"])
      except: nex = 1.0
      estnoise = 1
      print "Estimating noise."
      sky = f.row(0)
      nspec = sqrt(adu*(gspec + nex*sky) + pow(rn,2)*nex)/adu * greater(sky,0)
    gspecs = [gspec]
    nspecs = [nspec]
elif imacs:

    if naxis3 is 6: q = 6
    elif naxis3 is 5: q = 5
    else: raise "not a valid naxis3 size"
    gspecs = f.data()[q-2] # fluxed spectra
    nspecs = f.data()[q-1] # fluxed noise
    #qdump("nspec.fits",nspecs)
    #qdump("gspec.fits",gspecs)
    estnoise = 0
    #sys.exit()

else:
    raise "Select data source!" 

gcrval1 = float(f["crval1"])
gcdelt1 = float(f["cdelt1"])
gcrpix1 = float(f["crpix1"])
gdcflag = int(f["dc-flag"])
gobjs = [f["obj%03d" % (d+1)] for d in arange(len(gspecs))]
f.close()

CnWave = CnWaveDict[gcdelt1]

if sngle: Statname = input.replace(".dict",".out")
else: Statname = inspec.replace("1spec.fits","redshiftCC.out")
RedshiftStat = open(direct + "/" + Statname,"w")
RedshiftStat.write("Rows= %i\n" % (len(gobjs)))
#print sys.argv

tot = 0

if agn: ttype = "agn"
elif star: ttype = "star"
elif galaxy: ttype = "galaxy"

if input:
    objD = eval(open(input).read())
    objects = [obj for obj in objD if objD[obj]["status"]=="run"]
else:
    argv,rstart= rdarg(argv,"-rstart",int,0)
    argv,rend= rdarg(argv,"-rend",int,len(gspecs))
    objects = range(rstart,rend)

for k in objects:
    try: gobj = gobjs[k]
    except IndexError:
        print "Row %i not found in spectra!" % k
        continue
    gspec = gspecs[k]
    nspec = nspecs[k]
    print "\nComputing redshift for",inspec,"row",k,"of",len(gspecs)-1,"\n\n"
        
    nspec = nspec * VTKErode(greater(nspec,0)*1.0,5,1,numret=1)
    if uniform: nspec = mean(compress(greater(gspec,0),nspec))*ones(nspec.shape)
    
    gwave = (arange(len(gspec))+1-gcrpix1)*gcdelt1+gcrval1
    if gdcflag: gwave = power(10,gwave)
    
    lredshift = []
    if w1==w2:
        try:
            w1,w2 = take(gwave,extrema(nonzero(nspec)))
            print "w1,w2=(%f,%f)" % (w1,w2)
        except:
            lredshift.append({"error":"Error with object"})
            print "Error with object\n"

    umask=[[5570,5585]]
    if input:
        #if objD[k]["bin"]=="normal": ebin = 0; efit = 0; efind = 0.0
        if objD[k]["bin"]=="ebin": ebin = 1; efit = 1; efind = 3.0; esig = 2.0; efilt = 15.0
        templates = objD[k]["tplate"]
        ttype = objD[k]["ttype"]
        if objD[k]["mask"]:
            for i in objD[k]["mask"]: umask.append(i)
    else:
        templates = TemplateIDs[ttype]
        for i in range( argv.count("-mask")):
            argv,ums=rdarg(argv,'-mask')
            umask.append(map(int,split(ums,",")))
    masks = [amask,bmask,emask,umask]

    TemplateData = {}
    for t in templates:
        template = "%s/spDR2-%03d.fit" % (TemplatePath,t)
        f = FITS(template)
        tspec = f.row(trow)
        tcrval1 = float(f["crval1"])
        tcdelt1 = float(f["cdelt1"])
        tcrpix1 = float(f["crpix1"])
        tdcflag = int(f["dc-flag"])
        twave = (arange(len(tspec))+1-tcrpix1)*tcdelt1+tcrval1
        if tdcflag: twave = power(10,twave)
        if f["velocity"] != "N/A":
           tvel = float(f["velocity"])
           twave = twave / (1+tvel/CLIGHTK)
        if t in [23,24,25,26,27]: twave,tspec = compress(less(twave,8999),[twave,tspec],1)
        elif t in [29]: twave,tspec = compress(less(twave,7808),[twave,tspec],1)
        TemplateData[t]={"twave":twave,"tspec":tspec}
        f.close()

    for t in templates:
        t0 = time.time()

        template = "spDR2-%03d.fit" % (t)
        tspec = TemplateData[t]["tspec"]
        twave = TemplateData[t]["twave"]
        if input and objD[k]["z"] != 0.0: redshifts = [(objD[k]["z"],0.1,0)]
        else: redshifts = RedshiftBins[ttype]

        for guess,zrange,tnz in redshifts:
            ### Need to make self-cleaning mechanism to remove failed fits and exception postscript files
            output = "%04d_%s_cc_t%02d_z%.2f" % (k,gobj.strip(),t,guess)
            output1 = direct + "/"+output+".ps"
            output2 = direct + "/"+output+".bin"
            if not watch: watchdev = direct + "/%s.ps/vcps" % (output)

            gfilt,bfilt,ffilt,pfilt = FilterParams[ttype]
            if ebin:
                nz = zrange / ((2*gcdelt1/CnWave)*(1+guess))
                print "Binning=",nz

            if not tnz: tnz = nz
      
            print "File= %s" % (inspec)
            print "Template= spDR2-%03d.fit" % (t)
            print "Row= %i" % (k)
            print "Obj= %s" % (gobj)
            print "z= %.2f\n" % (guess)

            pid=os.fork()

            #print pid
            if pid==0:
                error = None

                ### Possible convergence problem with RedshiftInfo
                try: RedshiftInfo = GetRedshift(gwave,gspec,nspec,twave,tspec,w1,w2,guess,zrange,\
                      tnz,masks,gfilt=gfilt,bfilt=bfilt,ffilt=ffilt,pfilt=pfilt,tk=tk,efit=efit,\
                      efilt=efilt,citer=citer,efind=efind,trlim=trlim,inspec=inspec,\
                      template=template,inctitle=inctitle,estnoise=estnoise,rescale=rescale,\
                      prompt=prompt,pgdev=watchdev,pgsize=pgsize,pu=pu,dps=dps,tps=tps,dump=dump)
                except:
                    error = "Exception"
                    print "\nException\n"
                    RedshiftInfo = {}
                    
                if inter: raw_input("< hit cr> ")
                tf = open(output2,"wb")
                if not RedshiftInfo:
                   print "\nFit failed\n"
                   if error == "Exception": error = "Exception fit failed" 
                   else: error = "Fit Failed"
                for key in ["t","template","ttype","guess","zrange","tnz","masks","w1","w2","error","gobj","k"]:
                    RedshiftInfo[key] = eval(key)
                pickle.dump(RedshiftInfo,tf)
                tf.close()
                os._exit(0)

            else:
                os.waitpid(pid, 0)
                try: 
                    temp = pickle.load(open(output2))
                except: 
                    temp = {"error":"Problem opening %s" % (output2)}
                    print "\nProblem opening %s\n" % (output2)
                    if os.path.exists(output1): os.remove(output1)
                    if os.path.exists(output2): os.remove(output2)
                lredshift.append(temp)

        t1 = time.time()
        tot = tot + t1 - t0

    head = "Obj= %s Row= %i File= %s" % (gobj.strip(),k,inspec)
    top = "%3s %8s %7s %7s %10s %8s %8s %8s %6s %6s %s" % ("T","z","e1","e2","CC","rchi2","w1","w2","z_g","z_gr","elines")
    fmt = "%3d %8.5f-%.5f+%.5f %10.1f %8.3f %8.2f %8.2f %6.2f %6.2f %r"
    efmt = "%3d  %-43s %8.2f %8.2f %6.2f %6.2f"
    print head
    print top
    RedshiftStat.write("\n"+head+"\n")
    RedshiftStat.write("masks= %r\n" % (masks))
    RedshiftStat.write("nz= %r\n\n" % (nz))
    RedshiftStat.write(top+"\n")
    for redshift in lredshift:
        if redshift["error"]:
            if not redshift.has_key("t"): RedshiftStatv = redshift["error"]
            else: RedshiftStatv = efmt % tuple([redshift[key] for key in ["t","error","w1","w2","guess","zrange"]])
        else:
            if redshift["ccpeak"] > 99999999.9: redshift["ccpeak"] = 99999999.9
            if redshift["ccpeak"] < -9999999.9: redshift["ccpeak"] = -9999999.9
            if redshift["chi2"] > 9999.999: redshift["chi2"] = 9999.999
            RedshiftStatv = fmt % tuple([redshift[key] for key in ["t","redshift","rederr1","rederr2","ccpeak","chi2","w1","w2","guess","zrange","lnhd"]])
        print RedshiftStatv
        RedshiftStat.write(RedshiftStatv + "\n")

tail = "\nTotal redshift computation time done in %.2fs\n\n" % (tot)
print tail
RedshiftStat.write(tail)
RedshiftStat.close()
