#!/usr/bin/env python

from basics import *
from FITS import *
from pgplot import *
from sets import Set
from CosmosTools import bin_dict,bin_list,dict_update
from VTKHelperFunctions import *
import glob,sys

def cmd_help():
    print
    print "-1d           FITSNAME     1d spectra fits file name, required to open imacsDisplay.py"
    print "-big          FITSNAME     2d spectra fits file name, required to open imacsDisplay.py"
    print "-bin          REDSHIFTBIN  binary file name to create/load default is ccredshift.bin"
    print "-d            REDSHIFTDIR  redshift directory default is redshiftCC"
    print "-dict         DICT         python dictionary to load"
    print "-fr           FRAMENUM     frame number to start from default is 0" 
    print "-h    --help               print program options"
    print "-l            LINELIST     galaxy line list, default $PYTHONHOME1/linelists/galaxylines.dat"
    print "-t            TRACEBIN     trace, required to open imacsDisplay.py"
    print

def khelp():
    print "\nBasic"
    print "  q\t\tquit\n  r\t\treset zoom and stretch\n  Shift+;\tpython command\n  Shift+/\thelp\n  Space\t\tdisplay cursor position\n  Shift+d\tfork imacsDisplay.py\n  Shift+c\tadd comments\n  Shift+`\tdisplay spectra status\n  Shift+s\tsave to file\n  Shift+,\tsmaller pgplot window\n  Shift+.\tlarger pgplot window" 

    print "\nFrame"
    print "  n\t\tnext object\n  p\t\tprevious object\n  m\t\tmark spectra\n  f\t\tnext marked object\n  b\t\tprevious marked object"

    print "\nZoom"
    print "  Shift+x\tzoom in x\n  x\t\tzoom out in x\n  Shift+y\tzoom in y\n  y\t\tzoom out in y\n  Shift+z\tzoom in x and y\n  z\t\tzoom out in x and y\n  h\t\tpan to -x\n  j\t\tpan to -y\n  k\t\tpan to +y\n  l\t\tpan to +x"
    print "\nSmoothing"
    print "  Shift+g\tGaussian smoothing"

    print "\nCross-correlation"
    print "  c\t\tcross-correlation scale toggle\n  w\t\tpan to +Cw(z)\n  s\t\tpan to -Cw(z)\n  a\t\tzoom out in Cw(z)\n  Shift+a\tzoom in Cw(z)" 

    print "\nTemplate"
    print "  ,\t\tprevious template\n  .\t\tnext template"

    print "\nMask"
    print "  Shift+m\tmask display toggle"

    print "\nRedshift"
    print "  Shift+0\tredshift display toggle\n  [\t\tincrement in z -0.01 \n  ]\t\tincrement in z +0.01\n  Shift+[\tfine increment in z -0.0001\n  Shift+]\tfine increment in z +0.0001\n  Shift+1\tz set to 0.00\n  Shift+2\tz set to 0.10\n  Shift+3\tz set to 0.35\n  Shift+4\tz set to 0.75\n  Shift+5\tz set to 1.50\n  Shift+6\tz set to 3.00\n  Shift+7\tz set to 5.00"

class SingleCC:
    def __init__(self,data,mark,w1,w2,i1,i2,edata,zt,stat,smo):
        self.data = data
        self.mark = mark
        self.edata = edata
        self.stat = stat
        self.redshift = [self.edata["redshift"]["data"],self.edata["redshift"]["status"]]
        self.mask = self.edata["mask"]["status"]
        self.cws = self.edata["cw"]["status"]

        self.credshift = self.data["credshift"]
        self.ccpeak = self.data["ccpeak"]
        self.cc = self.data["cc"]
        self.chi2 = self.data["chi2"]
        self.obj = self.data["gobj"]
        self.k = self.data["k"]
        self.lnhd = self.data["lnhd"]
        self.m = self.data["masks"][3] 
        self.guess = self.data["guess"]
        self.rederr1 = self.data["rederr1"]
        self.rederr2 = self.data["rederr2"]
        self.t = self.data["t"]
        self.zrange = self.data["zrange"]
        self.matched = self.data["matched"]
        self.waves = self.matched[0]

        self.cwmin = self.edata["cw"]["min"]["t%02d" % (self.t)]
        self.cwmax = self.edata["cw"]["max"]["t%02d" % (self.t)]
        self.smo = smo

        if w1: self.w1 = w1
        else: self.w1 = self.matched[0][0]
        if w2: self.w2 = w2
        else: self.w2 = self.matched[0][-1]
        self.zt = zt
        if i1: self.i1 = i1
        else: self.i1 = -1*getpct(self.matched[1],98)
        if i2: self.i2 = i2
        else: self.i2 = 2*getpct(self.matched[1],98)
        self.xr = arange(self.w1,self.w2)
        self.yr = arange(self.i1,self.i2)
        self.yarr = None
        try:
            self.xarr = extrema(self.xr)
            self.yarr = extrema(self.yr)
        except:
            #if not row: row = 0
            #print "problem with spectra %i" % (row)
            print "problem with spectra"
    def GetInfo(self):
        return self.w1,self.w2,self.i1,self.i2,self.waves
    def Display(self):
        self.DisplayHisto()
        self.DisplayInfo()
        self.DisplaySpectra()
    def DisplaySpectra(self):
        cpgsvp(0.10,0.90,0.10,0.55)
        newbox([self.w1,self.w2],[self.i1,self.i2],manvp=1,ch=0.7,era=0)
        g = self.matched[1]
        if self.smo:
            if self.smo.values()[0] > 0: 
               if self.smo.keys()[0] == "B":
                   print "boxcar feature still under construction..."
                   wrttext("","(Boxcar fw="+str(self.smo.values()[0])+")",-35.0,0.6,0.8,"T",1,0,1)
               if self.smo.keys()[0] == "G":
                   g = VTKGauss(g,smo.values()[0]/2.35,numret=1)
                   wrttext("","(Gauss fw="+str(self.smo.values()[0])+")",-35.0,0.6,0.8,"T",1,0,1)
        #drawln(self.waves,self.matched[1],ls=1,ci=1)
        drawln(self.waves,g,ls=1,ci=1,hist=1)
        drawln(self.waves,self.matched[2],ls=1,ci=4,hist=1)
        drawln(self.waves,self.matched[3],ls=1,ci=2,hist=1)
        drawln(self.waves,self.matched[4],ls=1,ci=3,hist=1)
        xlabel("Observed Wavelength (\A)",ch=1.0)
        ylabel("Counts",ch=1.0)
        if self.m: self.DisplayM()
        if self.redshift[1]: self.DisplayRedshift()
        if self.stat: self.DisplayStatus()
    def DisplayHisto(self):
        cpgsvp(0.10,0.90,0.60,0.85)
        z1 = self.cc[0][0]
        z2 = self.cc[0][-1]
        if self.cws:
            c1 = 0.5*self.cwmin
            c2 = 1.2*self.cwmax
        else:
            c1 = getpct(self.cc[1],2)
            c2 = 1.5*getpct(self.cc[1],98)
        newbox([z1,z2],[c1,c2],manvp=1,ch=0.7)
        drawln(self.cc[0],self.cc[1],ls=1,ci=1,hist=1)
        if self.cws: wrttext("","C",0.0,0.5,0.0,"T",1,0,1)
        xlabel("\\fiz",ch=1.0)
        ylabel("\\fiC\uw\d(z)",ch=1.0)
    def DisplayInfo(self):
        cpgsvp(0.10,0.90,0.90,0.95)
        wrttext("","Object = "+self.obj,2.0,0.7,0.0,"T",1,0,1)
        wrttext("","Frame = "+str(self.k),2.0,0.7,0.25,"T",1,0,1)
        wrttext("","Template = "+str(self.t),2.0,0.7,0.45,"T",1,0,1)
        wrttext("","z Guess = "+str(self.guess),2.0,0.7,0.65,"T",1,0,1)
        if self.mark: wrttext("","Status = Marked",0.8,0.7,0.0,"T",1,0,1)
        if self.redshift[1] : wrttext("","z = %.5f" % (self.zt),2.0,0.7,0.85,"T",1,0,1)
        #if self.edata["dict"]: wrttext("","Status = "+self.S,0.8,0.7,0.0,"T",1,0,1)
        #wrttext("","z Range = "+str(self.zrange),2.0,0.7,0.5,"T",1,0,1)

        wrttext("","Redshift Peak = %10.8f -%.8f +%.8f" % (self.credshift,self.rederr1,self.rederr2),-1.8,0.8,0.0,"T",1,0,1)
        wrttext("","Peak Value = %-20.6f" % (self.ccpeak),-3.0,0.8,0.0,"T",1,0,1)
        wrttext("","Reduced \gx\u2\d = %-20.6f" % (self.chi2),-4.2,0.8,0.0,"T",1,0,1)

        #wrttext("","elines = "+str(self.lnhd),-3.0,0.6,0.0,"T",1,0,1)
    def DisplayM(self):
        if self.mask:
            for m in self.m:
                xm = array([m[0],m[1],m[1],m[0],m[0]])
                ym = array([self.i1,self.i1,self.i2,self.i2,self.i1])
                drawbox(xm,ym,5,1,1,3)
                drawbox(xm,ym,5,1,1,2)
    def DisplayRedshift(self):
        col=[0,4,2,1]
        wlist = self.redshift[0]
        w = []; a = []; s = []; v = []
        if self.yarr:
            for wl in wlist:
                w.append(float(wl[0])*(1+self.zt))
                a.append(col[int(wl[1])])
                s.append(wl[2])
                if self.waves[0] < w[-1] < self.waves[-1]:
                   specval = self.matched[1][searchsorted(self.waves,w[-1])]
                   specval = 0.15*(self.yarr[-1] - specval) + specval
                   v.append(specval)
                else:
                   v.append(-999)
            [drawln(w[j]*ones(self.yarr.shape),self.yarr,ls=4,ci=a[j]) for j in range(len(w))]
            drawptl(w,v,s,ch=0.7,ci=a,lw=1,ang=90,fj=0)
    def DisplayStatus(self):
        wrttext("",str(self.stat),-35.0,0.6,0.0,"T",1,0,1) 
   
ph1 = os.getenv("PYTHONHOME1")
if not ph1: raise "PYTHONHOME1 environmental variable not set!"

argv,chelp1 = rdarg(argv,"-h",None,0,single=1)
argv,chelp2 = rdarg(argv,"--help",None,0,single=1)
if chelp1 or chelp2:
    cmd_help()
    sys.exit()
size=8
opendev("/xs",nx=1,ny=1,size=size,ask=0)
argv,binary = rdarg(argv,"-bin",None,"ccredshift.bin")
argv,direct = rdarg(argv,"-d",None,"redshiftCC")
argv,fr = rdarg(argv,"-fr",int,0)
argv,spec1d = rdarg(argv,"-1d",None,None)
argv,spec2d = rdarg(argv,"-big",None,None)
argv,dict = rdarg(argv,"-dict",None,None)
argv,trace = rdarg(argv,"-t",None,None)
argv,wavefile = rdarg(sys.argv,"-l",None,"/".join([ph1,"linelists/galaxylines.dat"]))


bin = bin_list(direct,binary)
#bin = [int(j) for j in Set([l.replace(direct + "/","").split("_")[0] for l in glob.glob(direct + "/*.bin") if l.replace(direct + "/","") != binary])]
#bin.sort()

#objB = {}
#Template={"credshift":None,"ccpeak":None,"cc":None,"chi2":None,"gobj":None,"lnhd":None,"guess":None,"rederr1":None,"rederr2":None,"t":None,"zrange":None,"matched":None,"bin":None,"comments":None,"check":None}

klist = ["credshift","ccpeak","cc","chi2","gobj","lnhd","guess","rederr1","rederr2","t","zrange","matched"]

### Need condition if objects are added or removed from the dictionary objB!!!
### Old .bin with new or missing objects will bomb!!!

if dict and os.path.exists(dict):
    objD = eval(open(dict).read())
    objD = dict_update(objD,dict,spec1d)

objB = bin_dict(direct,binary,bin,objD,spec1d)
N = len(objD)

#if binary and os.path.exists(binary):
#    objB = pickle.load(open(binary))
#    for l in range(N): 
#        #objB[l]["comments"] = objD[l]["comments"] 
#        if [b for b in bin if b == l]:
#            blist = [h for h in glob.glob(direct + "/%04d*.bin" % (l)) if not pickle.load(open(h))["error"]]
#            blist.sort()
#            if blist: objB[l]["check"] = "ok"
#            else: objB[l]["check"] = "fail"
#        elif objD[l]["status"] == "skip": objB[l]["check"] = "skip"
#        for td in Template:
#            if not objB[l].has_key(td): objB[l][td] = Template[td]
#else:
#    #for l in range(N): objB[l]={"credshift":None,"ccpeak":None,"cc":None,"chi2":None,"gobj":None,"lnhd":None,"guess":None,"rederr1":None,"rederr2":None,"t":None,"zrange":None,"matched":None,"bin":None,"comments":objD[l]["comments"],"check":None}
#    for l in range(N): objB[l]={"credshift":None,"ccpeak":None,"cc":None,"chi2":None,"gobj":None,"lnhd":None,"guess":None,"rederr1":None,"rederr2":None,"t":None,"zrange":None,"matched":None,"bin":None,"check":None}
#    for l in range(N):
#        if [b for b in bin if b == l]:
#            blist = [h for h in glob.glob(direct + "/%04d*.bin" % (l)) if not pickle.load(open(h))["error"]]
#            blist.sort()
#            if blist: objB[l]["check"] = "ok"
#            else: objB[l]["check"] = "fail"
#        elif objD[l]["status"] == "skip": objB[l]["check"] = "skip"
#
if wavefile and os.path.exists(wavefile):
    wlines = open(wavefile,"r").readlines()
    wlines = filter(lambda l: l[0] != "#", wlines)
    wlist = map( split, wlines)
else:
    wlist = None

edataD = {}
edataD["redshift"] = {"data":wlist,"status":0}
edataD["mask"] = {"status":0}
edataD["cw"] = {"status":0,"min":{},"max":{}}

rindex = 0
if fr: rindex = bin.index(fr); r = bin[rindex]
else: r = bin[rindex]

rold,told = -999,-999

tindex = 0
key = '0'
xc=0
yc=0
data = None
mark = False
mask = False
tbin = None
newobj = True
stat = None
smo = {}
nsmo = {}
tsmo = 0

w1=0
w2=0
i1=0
i2=0
zf = 2.0
numkeys = ("0","1","2","3","4","5","6","7","8","9")

while key != "q":
    if newobj:
        tlist = [h for h in glob.glob(direct + "/%04d*.bin" % (r)) if not pickle.load(open(h))["error"]]
        tlist.sort()
        if not tlist:
            print "Fits failed for %i" % (r) 
            if rold > r: rindex -= 1
            else: rindex += 1
            rold = r
            r=bin[rindex]
            continue
        ts = [j for j in Set([h.replace(direct,"").split("_")[3] for h in tlist])]
        for t in ts:
            cw = [extrema(pickle.load(open(h))["cc"][1]) for h in tlist if t in h]
            edataD["cw"]["min"][t] = sort([c[0] for c in cw])[0]
            edataD["cw"]["max"][t] = sort([c[1] for c in cw])[-1]

    newobj = False
    if tbin: data = pickle.load(open(tbin)); tindex = tlist.index(tbin); tbin = None
    else: data = pickle.load(open(tlist[tindex]))
    
    tn = len(tlist)
    #print data
    #print r,tlist[tindex],tn

    if rold != r or told != tindex: zt = data["credshift"]
    #print r,rold,rindex

    if objB[r]["bin"] == tlist[tindex]: mark = True
    else: mark = False

    ObjCC = SingleCC(data,mark,w1,w2,i1,i2,edataD,zt,stat,smo)
    ObjCC.Display()
    w1,w2,i1,i2,waves = ObjCC.GetInfo()
    stat = None
    t = data["t"]
    status,xc,yc,key=cpgcurs(xc,yc)
    xran = (w2 - w1)
    yran = (i2 - i1)
    if key in numkeys:
        if nsmo.keys() and tsmo: smo = {nsmo.keys()[0]:int(key)}; tsmo = 0

    if key == "c":
        fcw = edataD["cw"]["status"]
        if fcw: fcw = 0
        else: fcw = 1
        edataD["cw"]["status"] = fcw
    if key == "~":
        print "\n  %3s %12s %4s %6s  %s" % ("row","object","mark","status","comments")
        print "-"*40
        for l in range(N):
            if objD[l]["comments"]: comments = objD[l]["comments"]
            else: comments = ""
            if objB[l]["bin"]: marked = "M"
            else: marked = ""
            print "%5i %12s %4s %6s  %s"  % (l,objD[l]["object"].strip(),marked,objB[l]["check"],comments)
        print
    if key=="S":
        f = open(binary,"w")
        pickle.dump(objB,f)
        f.close()
        f = open(dict,"w")
        f.write(str(objD))
        f.close()
        stat = "Files saved."
    if key == "C":
        if objD[r]["comments"]: print "\nPrevious comments: \n%s" % (objD[r]["comments"])
        temp = raw_input("\nEnter comments? (CR unchanged) ")
        if temp: objD[r]["comments"] = temp
    if key == "r": w1,w2,i1,i2,zt,newobj = 0,0,0,0,data["credshift"],True
    if key == "x" or key == "z":
        xran = xran*zf; w1 = xc - xran/2.; w2 = xc + xran/2.
        if w1 < waves[0]: w1 = waves[0]
        if w2 > waves[-1]: w2 = waves[-1]
    if key == "X" or key == "Z":
        xran = xran/zf; w1 = xc - xran/2.; w2 = xc + xran/2.
        if w1 < waves[0]: w1 = waves[0]
        if w2 > waves[-1]: w2 = waves[-1]
    if key == "y" or key == "z":
        yran = yran*zf
        i1 = yc - yran/2.; i2 = yc + yran/2.
    if key == "Y" or key == "Z":
        yran = yran/zf
        i1 = yc - yran/2.; i2 = yc + yran/2.
    if key == "h":
        w1 = w1 - xran/10.
        if w1 < waves[0]: w1 = waves[0]
        w2 = w1 + xran
    if key == "l":
        w2 = w2 + xran/10.
        if w2 > waves[-1]: w2 = waves[-1]
        w1 = w2 - xran
    if key == "j" or key == "k":
        if key =="j": direc = -1
        if key =="k": direc = +1
        i1 = i1 + direc*yran/10.; i2 = i2 + direc*yran/10.

    if key == "m":
        if tlist[tindex]==objB[r]["bin"]:
            for key in klist: objB[r][key] = None
            objB[r]["bin"]=None
        else:
            for key in klist: objB[r][key] = data[key]
            objB[r]["bin"]=tlist[tindex]

    if key == "B" or key == "G":
        if smo:
            if smo.keys()[0] != key: nsmo = {key:0}
        else:
            if key == "B": nsmo = {"B":0}
            if key == "G": nsmo = {"G":0}
        stat = "Press a number."
        tsmo = 1

    if key == "M":
        mask = edataD["mask"]["status"]
        if mask: mask = 0
        else: mask = 1
        edataD["mask"]["status"] = mask

    if key == "D":
        cmd = " ".join(["imacsDisplay.py","-1d",spec1d,"-big",spec2d,"-dict",dict,"-t",trace,"-fr",str(r),"-d",direct,"-p","/xw"])
        print cmd
        #print "How far am I(1)"
        os.system(cmd)
        #print "How far am I(2)"
        objD = eval(open(dict).read())
        #print "How far am I(3)"

    if key == "!": zt = 0.00
    if key == "@": zt = 0.10
    if key == "#": zt = 0.35 
    if key == "$": zt = 0.75
    if key == "%": zt = 1.50
    if key == "^": zt = 3.00
    if key == "&": zt = 5.00
    if key == "<":
        size -=2
        if size < 2: size = 2
        closedev()
        opendev("/xs",nx=1,ny=1,size=size,ask=0)
    if key == ">":
        size +=2
        if size > 14: size = 14
        closedev()
        opendev("/xs",nx=1,ny=1,size=size,ask=0)
    if key in ["[","]","{","}"] and wlist and edataD["redshift"]["status"]:
        if key == "[": zt = zt - 0.01
        if key == "]": zt = zt + 0.01
        if key == "{": zt = zt - 0.0001
        if key == "}": zt = zt + 0.0001

    #############################################################################################
    #
    # Should I loop with the marked spectra and/or unmarked???  i.e. When going over the last value
    # in the list start at the beginning or stop at the end.
    #
    #############################################################################################

    if key == "f":
        mos = [d for d in objB if objB[d]["bin"]]
        mbs = [objB[d]["bin"] for d in objB if objB[d] if objB[d]["bin"]]
        max = len(mos)-1    
        for d in mos:
            ind = mos.index(d)
            if ind == max and r == d: rindex = bin.index(mos[0]); tbin = mbs[0]; w1,w2,i1,i2 = 0,0,0,0; break
            if ind < max: 
                if r < mos[0] or r > mos[-1]: rindex = bin.index(mos[0]); tbin = mbs[0]; w1,w2,i1,i2 = 0,0,0,0; break
                if (r > d and r < mos[ind+1]) or r == d: rindex = bin.index(mos[ind+1]); tbin = mbs[ind+1]; w1,w2,i1,i2 = 0,0,0,0; break
    if key == "b":
        mos = [d for d in objB if objB[d]["bin"]]
        mbs = [objB[d]["bin"] for d in objB if objB[d] if objB[d]["bin"]]
        max = len(mos)-1    
        for d in mos:
            ind = mos.index(d)
            if ind == 0 and r == d: rindex = bin.index(mos[-1]); tbin = mbs[-1]; w1,w2,i1,i2 = 0,0,0,0; break
            if ind > 0: 
                if r < mos[0] or r > mos[-1]: rindex = bin.index(mos[-1]); tbin = mbs[-1]; w1,w2,i1,i2 = 0,0,0,0; break
                if (r < d and r > mos[ind-1]) or r == d: rindex = bin.index(mos[ind-1]); tbin = mbs[ind-1]; w1,w2,i1,i2 = 0,0,0,0; break
    if key == "n":
        if r!=bin[-1]: rindex+=1; tindex=0; w1,w2,i1,i2 = 0,0,0,0; newobj = True
    if key == "p":
        if r!=bin[0]: rindex-=1; tindex=0; w1,w2,i1,i2 = 0,0,0,0; newobj = True

    told = tindex
    if key == ",":
        if tindex!=0: tindex-=1; w1,w2,i1,i2 = 0,0,0,0
    if key == ".":
        if tindex!=tn-1: tindex+=1; w1,w2,i1,i2 = 0,0,0,0

    if key in ("w","s","a","A"):
        cwmi = edataD["cw"]["min"]["t%02d" % (t)]
        cwma = edataD["cw"]["max"]["t%02d" % (t)]
        cwran = cwma - cwmi
        if key == "w": cwmi=cwmi+cwran/6.; cwma=cwma+cwran/6.
        if key == "s": cwmi=cwmi-cwran/6.; cwma=cwma-cwran/6.
        if key == "a": cwmi=cwmi-cwran/6.; cwma=cwma+cwran/6.
        if key == "A": cwmi=cwmi+cwran/6.; cwma=cwma-cwran/6.
        edataD["cw"]["min"]["t%02d" % (t)] = cwmi
        edataD["cw"]["max"]["t%02d" % (t)] = cwma

    if key == ")":
        redp = edataD["redshift"]["status"]
        if redp: redp = 0
        else: redp = 1
        edataD["redshift"]["status"] = redp

    if key == " ":
       print xc,yc

    if key == "?":
        khelp()

    if key == ":":
        try: exec(raw_input("<cmd?> "))
        except: pass

    #print w1,w2,i1,i2,xc,yc
    #print key

    rold = r

    r=bin[rindex]
    if rold != r:
        newobj = True

f = open(binary,"w")
pickle.dump(objB,f)
f.close()

f = open(dict,"w")
f.write(str(objD))
f.close()


closedev()

