#!/usr/bin/env python

from basics import *
# Order seems to matter for imports, need to remove basics Template
from Cheetah.Template import Template
from CosmosTools import bin_dict,bin_list,dict_update
import glob,os,time,sys
import pgplot as PG

def cmd_help():
    print "-bin   REDSHIFTBIN  binary file name to create/load default is ccredshift.bin"
    print "-d     REDSHIFTDIR  redshift directory default is redshiftCC"
    print "-dict  DICT         python dictionary to load"
    print "-smf   SMFFILE      smf file"
    print "-fits  FITFILE      1d spectra fits file, required for postscript and jpeg plots"
    print "-nojpg              jpeg and postscripts are not created, images in html are left intact"

def RedshiftAscii(dict,objB,objD,L_ids,L_ra,L_dec):
    f = open(dict.replace(".dict",".out"),"w")
    omin = 6
    pmin = 10 
    cmin = 12
    omax = sort([len(objD[j]["object"].strip()) for j in objB])[-1]
    if omax > omin: omin = omax
    if len([j for j in objB if objB[j]["credshift"]]) > 0:
        pmax = sort([len("%.6f" % (objB[j]["ccpeak"])) for j in objB if objB[j]["ccpeak"]])[-1]
        cmax = sort([len("%.6f" % (objB[j]["chi2"])) for j in objB if objB[j]["chi2"]])[-1]
        if pmax > pmin: pmin = pmax
        if cmax > cmin: cmin = cmax

    fmt1 = join(["%",str(omin),"s"],"")
    fmt2 = join(["%",str(pmin),"s"],"")
    fmt2f = join(["%",str(pmin),".6f"],"")
    fmt3 = join(["%",str(cmin),"s"],"")
    fmt3f = join(["%",str(cmin),".6f"],"")
    fmt4 = join(["%","12","s"],"")
    #     spaces +  z + err1 + err2 + ccpeak + chi2 + t + guess
    emp =      6 + 10 +   11 +   11 +   pmin + cmin + 8 +     6
    #print emp
    #print fmt1, fmt2, fmt2f, fmt3, fmt3f, fmt4

    f.write(" ".join(["# %3s" % ("row"),fmt1 % ("object"),fmt4 % ("ra"),fmt4 % ("dec"),"%10s %11s %11s" % ("redshift","error1","error2"),fmt2 % ("peak_value"),fmt3 % ("reduced_chi2"),"%8s %6s %10s %-s\n" % ("template","guess","fit_status","comments")]))
    for j in objB:
        data = objB[j]
        object = objD[j]["object"].strip()
        comments =  objD[j]["comments"]
        k = L_ids.index(object) 
        ra = L_ra[k]
        dec = L_dec[k]
        if not comments: comments = ""
        if data["credshift"]:
            f.write(" ".join(["%5i" % (j),fmt1 % (object),fmt4 % (ra),fmt4 % (dec),"%10.8f -%.8f +%.8f" % (data["credshift"],data["rederr1"],data["rederr2"]),fmt2f % (data["ccpeak"]),fmt3f % (data["chi2"]),"%8i %.4f %10s" % (data["t"],data["guess"],data["check"]),comments,"\n"]))
        else: 
            f.write(" ".join(["%5i" % (j),fmt1 % (object),fmt4 % (ra),fmt4 % (dec)," "*emp,"%10s" % (data["check"]),comments,"\n"]))
    f.close()
    print "%s complete" % (dict.replace(".dict",".out"))
 
def RedshiftCheetah(dict,objB,objD,L_ids,L_ra,L_dec,dir,objects):
    indent = " "*6
    templateDef = """
    <HTML>
    <HEAD><TITLE>$title</TITLE></HEAD>
    <BODY>
     
      <h2>$title</h2> 
      $info 
      $datatable
      $notes
    
    </BODY>
    </HTML>"""
    t = Template(templateDef,searchList=[])

    t.title = dict.replace("_redshift.dict","")
    t.info = "<P>The objects in this table were filtered from the original data set that were capable of determining a redshift for.  Objects not listed were filtered out due to severly poor sky subtraction, undesired objects such as holes, severe 0th order contaimination, and severe flattened flat fielding issues.</P><P>The templates used are from the Sloan Digital Sky Survey.</P>"

    tablebegin = "<P><FONT SIZE=2><TABLE BORDER=1 WIDTH=100% HEIGHT=100% ID='sortable' CLASS='listing'>\n"
    tablehead = indent + "<THEAD>\n" + indent + "<TR><TH>Row<TH>Object<TH>Plots<TH>RA<TH>Dec<TH>Redshift Peak<TH>Error1<TH>Error2<TH>Peak Value<TH>Reduced &Chi;<sup>2</sup><TH>Template<TH>Guess<TH>Fit Status<TH>Comments</TR>\n" + indent + "</THEAD>\n"

    tabletail = indent + "</TABLE></FONT></P>"
    tablebody = ""

    #objB_L = [objB[b]["gobj"] for b in objB]
    #objD_L = [objD[d]["object"] for d in objD]
    #print objB_L
    #print objD_L

    for j in objB:
        data = objB[j]
        object = objD[j]["object"].strip()
        comments = objD[j]["comments"]
        k = L_ids.index(object) 
        ra = L_ra[k]
        dec = L_dec[k]

        print j,object,objects[j]


        if comments and comments.strip(): cint = 1
        else: comments = "-"; cint = 0
        try:
            plot = "%s/%04d_%s.ps" % (dir,j,objects[j])
            img = "%s/%04d_%s.jpg" % (dir,j,objects[j])
 
            if data["credshift"]:
                tablebody = tablebody + indent + "<TR><TD align='center'>%i<TD align='center'><a href='%s'>%s</a><TD align='center'><a href='%s'><img src='%s'></a><TD align='center'>%s<TD align='center'>%s<TD align='center'>%10.8f<TD align='center'>-%.8f<TD align='center'>+%.8f<TD align='center'>%20.6f<TD align='center'>%20.6f<TD align='center'>%i<TD align='center'>%f<TD align='center'>%s<TD align='%s'>%s</TR>\n" % (j,data["bin"].replace("bin","ps"),object,plot,img,ra,dec,data["credshift"],data["rederr1"],data["rederr2"],data["ccpeak"],data["chi2"],data["t"],data["guess"],data["check"],["center","left"][cint],comments)
            else:
                tablebody = tablebody + indent + "<TR><TD align='center'>%i<TD align='center'>%s*<TD align='center'><a href='%s'><img src='%s'></a><TD align='center'>%s<TD align='center'>%s<TD align='center'>-<TD align='center'>-<TD align='center'>-<TD align='center'>-<TD align='center'>-<TD align='center'>-<TD align='center'>-<TD align='center'>%s<TD align='%s'>%s</TR>\n" % (j,object,plot,img,ra,dec,data["check"],["center","left"][cint],comments)
        except IndexError:
            print "Object %i not found!" % (j)

    if tablebody is "":
        t.datatable = ""
    else:
        t.datatable = tablebegin + tablehead + tablebody + tabletail
   
    t.notes = "<P>* Not able to determine redshift.</P>"
 
    content = t.respond()
    #print t 

    f = open("index.html","w")
    f.write(content)
    f.close()
    print "index.html complete"

def FitsPostscript(j,S,N,w,obj,dir):
    PG.opendev("%s/%04d_%s.ps/vcps" % (dir,j,obj),nx=1,ny=1,land=0,size=10,asp=0.8)
    PG.newbox([w[0],w[-1]],[-3*getpct(N,98),2*getpct(S,98)],manvp=1,ch=0.7,era=0)
    PG.wrttext("",obj,0.5,1.0,0.0,"T",1,0,1)
    PG.xlabel("Wavelength (\A)",ch=1.0)
    PG.ylabel("Counts",ch=1.0)
    PG.drawln(w,S,ls=1,ci=1)
    PG.drawln(w,N,ls=1,ci=2)
    PG.closedev()

def FitsJPG(j,obj,dir):
    os.system("convert -geometry 132x102 -quality 100 %s/%04d_%s.ps %s/%04d_%s.jpg" % (dir,j,obj,dir,j,obj))

argv,binary = rdarg(argv,"-bin",None,"ccredshift.bin")
argv,dict = rdarg(argv,"-dict",None,"redshift.dict")
argv,direct = rdarg(argv,"-d",None,"redshiftCC")
argv,smf = rdarg(argv,"-smf",None,None)
argv,fname = rdarg(argv,"-fits",None,None)
argv,nojpg = rdarg(argv,"-nojpg",None,0,single=1)
argv,dir = rdarg(argv,"-fits",None,"plots")
argv,chelp = rdarg(argv,"-h",None,0,single=1)
if chelp:
    cmd_help()
    sys.exit()
objD = eval(open(dict).read())
objD = dict_update(objD,dict,fname)
bin = bin_list(direct,binary)
objB = bin_dict(direct,binary,bin,objD,fname)
objects = None

L_all = open(smf,"r").readlines()
L = [l.split() for l in L_all]
L_ids = [l[1] for l in L if l[0] in ["HOLE","SLIT"]]
L_ra = [l[2] for l in L if l[0] in ["HOLE","SLIT"]]
L_dec = [l[3] for l in L if l[0] in ["HOLE","SLIT"]]

f = FITS(fname)
naxis3 = f["naxis3"]
n_slits = f["n_slits"]
w = (arange(f["naxis1"])+1.0-f["crpix1"])*f["cdelt1"] + f["crval1"]
if f["dc-flag"] is 1: w = power(10,w)
objects = [f["obj%03d" % (k)].strip() for k in 1+arange(n_slits)]
if naxis3 is 6: q = 6
elif naxis3 is 5: q = 5
else: raise "not a valid naxis3 size"
spectra = f.data()[q-2] # fluxed spectra
noise = f.data()[q-1] # fluxed noise
if not os.path.exists(dir): os.mkdir(dir)
if not nojpg: [FitsPostscript(j,spectra[j],noise[j],w,obj,dir) for j,obj in enumerate(objects)]
if not nojpg: [FitsJPG(j,obj,dir) for j,obj in enumerate(objects)]
f.close()

RedshiftCheetah(dict,objB,objD,L_ids,L_ra,L_dec,dir,objects)
RedshiftAscii(dict,objB,objD,L_ids,L_ra,L_dec)



