#!/usr/bin/env python

from FITS import *
from pgplot import *
from basics import *

class Plot:
    def __init__(self,FITS1,fmt,obj,fr,ffile):
        self.FITS1 = FITS1
        self.ffile = ffile
        self.fr = fr 
        self.fmt = fmt
        self.obj = obj
        if self.fmt == "ext":
            self.FITS1.goto(1)
        self.crval1 = self.FITS1["crval1"]
        self.cdelt1 = self.FITS1["cdelt1"]
        self.crpix1 = self.FITS1["crpix1"]
        if self.crpix1 is not FloatType: self.crpix1 = 1
        self.dcflag = self.FITS1["dcflag"]
        if self.dcflag is not IntType: self.dcflag = 0
        self.n_slits = self.FITS1["n_slits"]
        if self.fmt == "ext": 
            self.FITS1.goto(self.obj)
            self.cntrline = self.FITS1["cntrline"]
            self.slitnum = self.FITS1["slitnum"]
            self.object = self.FITS1["object"].strip()
        self.naxis1 = self.FITS1["naxis1"]
        self.naxis2 = self.FITS1["naxis2"]
        self.naxis3 = self.FITS1["naxis3"]

        #print self.naxis1, self.naxis2, self.naxis3

        self.data = self.FITS1.data()
        #print self.data.shape
        #print self.crval1, self.cdelt1, self.crpix1
        
        self.w = (arange(self.naxis1)+1-self.crpix1)*self.cdelt1+self.crval1
        self.c = arange(self.naxis2)
        if self.dcflag: self.w = power(10,self.w)

    def Display(self,w1,w2,i1,i2,pos,ori,ave):
        self.pos = pos
        self.ori = ori
        self.ave = ave
        #print self.data.shape
        #print self.data[0].shape
        #print self.data[0][0].shape
        #if ori == "COLUMN": self.data = transpose(self.data)
        #print self.data.shape
        if w1: self.w1 = w1
        else:
            if self.ori == "ROW": self.w1 = self.w[0]
            else: self.w1 = self.c[0]
        if w2: self.w2 = w2
        else: 
            if self.ori == "ROW": self.w2 = self.w[-1]
            else: self.w2 = self.c[-1]
        if i1: self.i1 = i1
        else: 
            if self.ori == "ROW":
                self.i1 = -2*abs(sum(self.data[::][self.fr][self.pos])/len(self.data[::][self.fr][self.pos]))
            else:
                self.i1 = -2*abs(sum(transpose(self.data[self.fr])[self.pos])/len(transpose(self.data[self.fr])[self.pos]))
        if i2: self.i2 = i2
        else: 
            if self.ori == "ROW":
                self.i2 = 4*abs(sum(self.data[::][self.fr][self.pos])/len(self.data[::][self.fr][self.pos]))
            else:
                self.i2 = 4*abs(sum(transpose(self.data[self.fr])[self.pos])/len(transpose(self.data[self.fr])[self.pos]))
        #print self.w1,self.w2,self.i1,self.i2
        self.Display1D()
    def Display1D(self):
        newbox([self.w1,self.w2],[self.i1,self.i2],manvp=1,ch=0.7)
        if self.ori == "ROW": xlabel("Wavelength (\A)",ch=1.0)
        else: xlabel("Row",ch=1.0)
        ylabel("Counts",ch=1.0)

        #if ave:
        #    self.data[::][k][self.pos]

        if self.ori == "ROW":
            for k in range(self.naxis3): drawln(self.w,self.data[::][k][self.pos],ls=1,ci=k+1,hist=1)
        else:
            for k in range(self.naxis3): drawln(self.c,transpose(self.data[k])[self.pos],ls=1,ci=k+1,hist=1)
        tab = " "*5
        if self.fmt == "ext":
            head = tab.join([self.ffile,"Slitnum = %i" % self.slitnum,"Object = %s" % self.object,"Cntrline = %i" % self.cntrline])
        else:
            head = self.ffile
        if self.ori == "COLUMN": head = tab.join([head,"Column = %i" % self.pos])
        else: head = tab.join([head,"Row = %i" % self.pos])
        wrttext("",head,1.0,0.8,0.0,"T",1,0,1)



fr = 0
pos = 0
ori = "ROW"
fmt = "equi"
obj = 1
pgdev = "/xs"
w1,w2,i1,i2 = 0,0,0,0
for n,arg in enumerate(sys.argv):
    if arg in ("-c","--column"): ori = "COLUMN"
    elif arg in ("-e","--ext"): fmt = "ext"
    elif arg in ("-f","--fits"): ffile = sys.argv[n+1]
    elif arg in ("-h","--help"):
        help()
        sys.exit()
    elif arg in ("-i1"): i1 = float(sys.argv[n+1])
    elif arg in ("-i2"): i2 = float(sys.argv[n+1])
    elif arg in ("-o","--obj"): obj = int(sys.argv[n+1])
    elif arg in ("-p","--pos"): pos = int(sys.argv[n+1])
    elif arg in ("-pg","--pgdev"): pgdev = sys.argv[n+1]
    elif arg in ("-r","--row"): ori = "ROW"
    elif arg in ("-w1"): w1 = float(sys.argv[n+1])
    elif arg in ("-w2"): w2 = float(sys.argv[n+1])

size = 8
opendev(pgdev,nx=1,ny=1,land=1,size=size,ask=0)
FITS1 = FITS(ffile)
zf = 2.0
w10,w20 = 1*w1,1*w2
i10,i20 = 1*i1,1*i2
xc,yc = 0,0
k = "0"
ave = 0
while k != "q":
    D = Plot(FITS1,fmt,obj,fr,ffile)
    D.Display(w1,w2,i1,i2,pos,ori,ave)
    status,xc,yc,k=cpgcurs(xc,yc)
    w1,w2 = D.w1,D.w2
    i1,i2 = D.i1,D.i2
    xran = (w2 - w1)
    yran = (i2 - i1)
    if k == "u":
        if ori == "ROW": max = D.naxis2-1
        else: max = D.naxis1-1
        if pos != max: pos+=1
    if k == "d": 
        if pos != 0: pos-=1
    if k == "n":
        if fmt == "ext": 
            omax = D.n_slits
            if obj != omax: obj+=1; w1,w2,i1,i2 = w10,w20,i10,i20
    if k == "p":
        if obj != 0: obj-=1; w1,w2,i1,i2 = w10,w20,i10,i20
    if k == "r": w1,w2,i1,i2 = w10,w20,i10,i20
    #if k == "A":
    #    ave = raw_input("Enter the half width of %ss to average (default 30)? " % (ori.lower()))
    #    if not ave: ave = 10
    #    ave = int(ave)
    if k == "x" or k == "z":
        xran = xran*zf; w1 = xc - xran/2.; w2 = xc + xran/2.
        if w1 < D.w[0]: w1 = D.w[0]
        if w2 > D.w[-1]: w2 = D.w[-1]
    if k == "X" or k == "Z":
        xran = xran/zf; w1 = xc - xran/2.; w2 = xc + xran/2.
        if w1 < D.w[0]: w1 = D.w[0]
        if w2 > D.w[-1]: w2 = D.w[-1]
    if k == "y" or k == "z":
        yran = yran*zf
        i1 = yc - yran/2.; i2 = yc + yran/2.
    if k == "Y" or k == "Z":
        yran = yran/zf
        i1 = yc - yran/2.; i2 = yc + yran/2.
    if k == "h":
        w1 = w1 - xran/10.
        if w1 < D.w[0]: w1 = D.w[0]
        w2 = w1 + xran
    if k == "l":
        w2 = w2 + xran/10.
        if w2 > D.w[-1]: w2 = D.w[-1]
        w1 = w2 - xran
    if k == "j" or k == "k":
        if k =="j": direc = -1
        if k =="k": direc = +1
        i1 = i1 + direc*yran/10.; i2 = i2 + direc*yran/10.

    #print obj,pos
closedev()
