#!/usr/bin/env python
#
# Flux spectra from flattened-flatfields.
#
# Last modified 3-8-06

import pgplot as PG
import sys
import time
import os
from VTKHelperFunctions import *
from MLab import *
from FITS import *
from Multipack import *
from basics import *
from FFT import *

def help():
    print
    print "Usage: imacsSpecFlux.py -f 1DSPECFITS -m SMF -o ORD -s1 NUM -s2 NUM [OPTION]..."
    print
    print "This routine takes the reference objects you selected and normalizes the"
    print "flat-flat by median of the flat-flat and then divides reference spectra by"
    print "the normalized flat-flat.  A polynomial is fit to the reference spectra which"
    print "should resemble a blackbody.  Then a cross-corelation is run between the"
    print "reference spectra and all of the object spectra to find a lag for each in"
    print "order to adjust the shifts of the spectra.  Next all of the spectra are divided"
    print "by the polynomial that fit the reference spectra."
    print
    print "-Ab1  --Aband1       WAVEVALUE          Lower wavelength range of Aband"
    print "-Ab2  --Aband2       WAVEVALUE          Upper wavelength range of Aband"
    print "-Bb1  --Bband1       WAVEVALUE          Lower wavelength range of Bband"
    print "-Bb2  --Bband2       WAVEVALUE          Upper wavelength range of Bband"
    #print "-bin                 FILENAME           import in fitted, absorb. spectra"
    print "-f    --fitsfile     FILENAME           1d fits file"
    print "-ff   --ffitsfile    FILENAME           1d fits file to use for slope"
    print "-h    --help                            print program options"
    print "-i    --interactive                     interactive plots"
    print "-l    --lag                             lag plots"
    print "-m    --mask         FILENAME           smf file"
    print "-mr   --maskreg      '[[REG11,REG12]]'  mask out additional bad regions"
    ######################################################################################
    ###
    ###   in the format:   --maskreg "[[3400,3600],[4390,4450],[5000,6000]]"
    ###
    ######################################################################################
    #print "-o    --order        ORDERNUM           order of polynomial"
    print "-pcc  --plotcc                          cross-correlation plots"
    #print "-s1   --slitnum1     OBJECTNUM          reference obj using absorp. features"
    #print "-s2   --slitnum2     OBJECTNUM          reference obj fit for slope"
    print "-Vb1  --VRedband1    WAVEVALUE          Lower wavelength range of VeryRedband"
    print "-Vb2  --VRedband2    WAVEVALUE          Upper wavelength range of VeryRedband"
    print "-Wb1  --Waterband1   WAVEVALUE          Lower wavelength range of Waterband"
    print "-Wb2  --Waterband2   WAVEVALUE          Upper wavelength range of Waterband"
    print
    print "The wavelength band ranges are the following:"
    print "  Aband = 7570,7700"
    print "  Bband = 6850,6960"
    print "  Waterband = 7140,7360"
    print "  VeryRedBand = 9270,9770"
    print

def lag_smf(ids,lags,fluxes,amps,smffile,lagtest):

    L = [l.split() for l in open(smffile,"r").readlines() if l.split()[0] in ["HOLE","SLIT"]]
    smf_id = [l[1] for l in L]
    smf_x, smf_y = transpose([map(float,L[smf_id.index(i)][4:6]) for i in ids])
    smf_t = array([["HOLE","SLIT"].index(L[smf_id.index(i)][0]) for i in ids])
    fth = median(compress(smf_t,fluxes))/10

    hlags, hflux, hamp, hsmf_x, hsmf_y = compress((1-smf_t), [lags, fluxes, amps, smf_x, smf_y], 1)
    ulags, uflux, uamp, usmf_x, usmf_y = compress(greater(fluxes,fth)*smf_t, [lags, fluxes, amps, smf_x, smf_y], 1)

    ulag0,ulag1 = bwt(ulags)
    use = less(abs(ulags-ulag0),5*ulag1)
    wt = getsqrt(uflux)

    for j in range(5):
      basis = [power(usmf_x,xo)*power(usmf_y,yo) for xo in range(4) for yo in range(4-xo)]
      sol = svdfit(transpose(basis)*(wt*use)[::,NewAxis],ulags*(wt*use))
      flags = dot(sol,basis)
      resid = flags-ulags
      b0,b1 = bwt(compress(use,resid))
      print j+1,b0,b1
      use = less(abs(resid-b1),[3,5][j>0]*b1)

    if lagtest:
        PG.opendev("/xs",nx=4,ny=2,land=0,asp=0.5,size=15)
        
        PG.newbox(extrema(smf_x),[0.0,2*getpct(uamp,90)],ch=2.2)
        PG.xlabel("X mask",ch=2.2)
        PG.ylabel("max(C)",ch=2.2)
        PG.drawpt(smf_x,amps,ps=17,ch=2.5,ci=1)
        PG.drawpt(usmf_x,uamp,ps=17,ch=2,ci=4)
        PG.drawpt(hsmf_x,hamp,ps=17,ch=2,ci=2)
        
        PG.newbox(extrema(smf_y),[0.0,2*getpct(uamp,90)],ch=2.2)
        PG.xlabel("Y mask",ch=2.2)
        PG.ylabel("max(C)",ch=2.2)
        PG.drawpt(smf_y,amps,ps=17,ch=2.5,ci=1)
        PG.drawpt(usmf_y,uamp,ps=17,ch=2.0,ci=4)
        PG.drawpt(hsmf_y,hamp,ps=17,ch=2.0,ci=2)
        
        PG.newbox(extrema(smf_x),[0.0,2*getpct(uflux,90)],ch=2.2)
        PG.xlabel("X mask",ch=2.2)
        PG.ylabel("flux",ch=2.2)
        PG.drawpt(smf_x,fluxes,ps=17,ch=2.5,ci=1)
        PG.drawpt(usmf_x,uflux,ps=17,ch=2,ci=4)
        PG.drawpt(hsmf_x,hflux,ps=17,ch=2,ci=2)
        
        PG.newbox(extrema(smf_y),[0.0,2*getpct(uflux,90)],ch=2.2)
        PG.xlabel("Y mask",ch=2.2)
        PG.ylabel("flux",ch=2.2)
        PG.drawpt(smf_y,fluxes,ps=17,ch=2.5,ci=1)
        PG.drawpt(usmf_y,uflux,ps=17,ch=2.0,ci=4)
        PG.drawpt(hsmf_y,hflux,ps=17,ch=2.0,ci=2)
        
        PG.newbox(extrema(smf_x),[-10,10],ch=2.2)
        PG.xlabel("X mask",ch=2.2)
        PG.ylabel("lag",ch=2.2)
        PG.drawpt(smf_x,lags,ps=17,ch=2.5,ci=1)
        PG.drawpt(usmf_x,ulags,ps=17,ch=2,ci=4)
        PG.drawpt(hsmf_x,hlags,ps=17,ch=2,ci=2)
        PG.drawpt(usmf_x,flags,ps=17,ch=1.5,ci=7)
        
        PG.newbox(extrema(smf_y),[-10,10],ch=2.2)
        PG.xlabel("Y mask",ch=2.2)
        PG.ylabel("lag",ch=2.2)
        PG.drawpt(smf_y,lags,ps=17,ch=2.5,ci=1)
        PG.drawpt(usmf_y,ulags,ps=17,ch=2.0,ci=4)
        PG.drawpt(hsmf_y,hlags,ps=17,ch=2.0,ci=2)
        PG.drawpt(usmf_y,flags,ps=17,ch=1.5,ci=7)
        
        PG.newbox(extrema(smf_x),[-10,10],ch=2.2)
        PG.xlabel("X mask",ch=2.2)
        PG.ylabel("\gDlag",ch=2.2)
        PG.drawpt(usmf_x,flags-ulags,ps=17,ch=2,ci=4)
        
        PG.newbox(extrema(smf_y),[-10,10],ch=2.2)
        PG.xlabel("Y mask",ch=2.2)
        PG.ylabel("\gDlag",ch=2.2)
        PG.drawpt(usmf_y,flags-ulags,ps=17,ch=2.0,ci=4)
        
        PG.closedev()

    basis = [power(smf_x,xo)*power(smf_y,yo) for xo in range(4) for yo in range(4-xo)]
    fitted = dot(sol,basis)

    new_lags = where(smf_t,fitted,lags)
    return new_lags


def Aconvolve(spec,fluxing,wavelengths,cdelt1,interactive,plotcc,k,slitnum,pix=10):
    Abandmin = 7570 - pix*cdelt1 
    Abandmax = 7700 + pix*cdelt1 

    Aband = between(wavelengths,Abandmin,Abandmax)
    nAband = sum(Aband)
    object_flux = sum(spec*Aband)
    if not object_flux: return -999,0.0,0.0

    specmax = max(spec*Aband)
    fluxingmax = max(fluxing*Aband)

    speclevel = ones(wavelengths.shape)*specmax
    fluxinglevel = ones(wavelengths.shape)*fluxingmax

    specfilter = where(spec,spec,speclevel)*Aband + speclevel*logical_not(Aband)
    fluxingfilter = fluxing*Aband + fluxinglevel*logical_not(Aband)

    specfilter = specfilter / median(specfilter)
    fluxingfilter = fluxingfilter / median(fluxingfilter)

    specfilter = specfilter - median(specfilter)
    fluxingfilter = fluxingfilter - median(fluxingfilter)

    nlen = len(fluxing)
    nlen2 = power(2,int(log(nlen)/log(2)) + 1)

    arrays = "fluxing","spec","fluxingfilter","specfilter"
    for a in arrays:
      exec("%s2 = zeros((nlen2,),Float64)" % (a))
      exec("%s2[:nlen] = %s" % (a,a))

    # Convolution
    r = fft(specfilter2)
    s = fft(fluxingfilter2)
    c = s*conjugate(r)
    c = inverse_fft(c).real.astype(Float32)
    c = VTKImageShift(c,-nlen2/2.0,0,numret=1,wrap=1)
    lc = getlog10(c)
#    qdump("cross_cor.fits",[c,specfilter2,fluxingfilter2])
    cmax = argmax(c[len(c)/2-50:len(c)/2+50]) + (len(c)/2-50)
    ccmax = max(c[len(c)/2-50:len(c)/2+50])
    lag = cmax - len(c)/2.0
    print "Nlen2=",nlen2
    print "CMax=",cmax
    print "Max(C)=",ccmax
    print "Lag=",lag

    c0 = int(cmax-5)
    c1 = int(cmax+6)
    c_x = arange(len(c))[c0:c1]
    c_y = lc[c0:c1]
    c_b = polynom(c_x,2)
    c_s = svdfit(transpose(c_b),c_y)
    c_f = dot(c_s,c_b)
    print c_s
    x_c = -0.5*c_s[1]/c_s[2]
    print "x_c=",x_c
    lag = x_c - len(c)/2.0
    print "Lag=",lag

    # Plot to screen
    if plotcc:
       arrays = "fluxing2","spec2","fluxingfilter2","specfilter2","c","lc"
       for ja,a in enumerate(arrays):
          ci = [2,3,2,3,4,4][ja]
          if a == "lc":
            exec('PG.newbox([cmax-15,cmax+15],extrema(%s[int(cmax-15):int(cmax+15)]),ch=2.2)' % (a))
            exec('PG.xlabel("x",ch=2.2)')
            exec('PG.ylabel("log(C)",ch=2.2)')
          else:
            exec('PG.newbox([0,len(%s)],extrema(%s),ch=2.2)' % (a,a))
            if a == "spec2": exec('PG.subtitle("Object, Slit="+str(k),ch=2)')
            if a == "specfilter2": exec('PG.subtitle("Filtered Object, Slit="+str(k),ch=2)')
            if a == "fluxing2": exec('PG.subtitle("Star, Slit="+str(slitnum),ch=2)')
            if a == "fluxingfilter2": exec('PG.subtitle("Filtered Star, Slit="+str(slitnum),ch=2)')
            exec('PG.xlabel("x",ch=2.2)')
          if a == "c": exec('PG.ylabel("C",ch=2.2)')
          else: exec('PG.ylabel("I",ch=2.2)')
          exec('PG.drawln(arange(0,len(%s)),%s,lw=1,ls=1,ci=%d)' % (a,a,ci))

       PG.drawln([len(c)/2.0,len(c)/2.0],[0.0,max(c)],ls=4,ci=1)
       PG.drawln([x_c,x_c],[0.0,max(c)],ls=2,ci=7)
       PG.drawln(c_x,c_f,ls=1,ci=1)

    f2 = VTKImageShift(fluxingfilter,lag,0,numret=1)
    b = polynom(f2,1)
    rescale = svdfit(transpose(b),specfilter)
    print rescale
    amplitude = rescale[1]

    if interactive: raw_input("<< cr >> ")
    return lag,object_flux,amplitude

def GetAmp(spec,fluxing,wavelengths,cdelt1,lag,pix=10,deltas=[-1,0,1]):

    Abandmin = 7570 - pix*cdelt1 
    Abandmax = 7700 + pix*cdelt1 

    Aband = between(wavelengths,Abandmin,Abandmax)
    nAband = sum(Aband)
    object_flux = sum(spec*Aband)
    if not object_flux:
      rescale = zeros((len(deltas),),Float32)
      rescale[len(rescale)/2] = 1.0
      return rescale

    specmax = max(spec*Aband)
    fluxingmax = max(fluxing*Aband)

    speclevel = ones(wavelengths.shape)*specmax
    fluxinglevel = ones(wavelengths.shape)*fluxingmax

    specfilter = where(spec,spec,speclevel)*Aband + speclevel*logical_not(Aband)
    fluxingfilter = fluxing*Aband + fluxinglevel*logical_not(Aband)

    specfilter = specfilter / median(specfilter)
    fluxingfilter = fluxingfilter / median(fluxingfilter)

    specfilter = specfilter - median(specfilter)
    fluxingfilter = fluxingfilter - median(fluxingfilter)

    b = [VTKImageShift(fluxingfilter,lag+d,0,numret=1) for d in deltas]
    rescale = svdfit(transpose(b),specfilter)
    return rescale


def fitshape(wavelengths,spec,noise,flat,atmos,order,interactive):
    nflat = divz(flat,median(flat))
    nspec = divz(spec,nflat)
    sspec = divz(noise,nflat)
    good = ones(wavelengths.shape) 
    for plots in range(2):
       if plots:
         newplot = "%s/vcps" % (fitsfile.replace("_2p_1spec.fits","_1spec_flux.ps"))
         uorders = [order]
       else:
         newplot = "/xs"
         uorders = range(max([order-2,2]),order+1)
       for uorder in uorders:
           rms_scatter = 1e38
           niter = 0
           while (len(uorders)>1 and ((niter < 3 or rms_scatter < rms_scatter0) and (niter < 15))) \
                or niter == 0:
               niter += 1
               if len(uorders)>1:
                  Xw = (wavelengths[-1] - wavelengths[0])/2.0
                  Xc = (wavelengths[-1] + wavelengths[0])/2.0
                  X = (wavelengths-Xc)/Xw
                  A = legendre(X,uorder)
                  weight = divz(good*logical_not(atmos).astype(Float32),sspec)
                  coefficients = linear_least_squares(transpose(A)*weight[::,NewAxis],nspec*weight)[0]
                  print "Iteration=",niter
                  print "Coefficients=",coefficients
                  fitted = dot(coefficients,A)
                  residuals = nspec - fitted
                  rms_scatter0 = rms_scatter
                  rms_scatter = sqrt(sum(weight*power(residuals,2))/sum(weight))
                  print "RMS=",rms_scatter
                  if niter < 4:
                      good = between(residuals,-2*rms_scatter,6*rms_scatter)

               # Plot fit
               if newplot:
                 PG.opendev(newplot,nx=2,ny=1,land=0,asp=0.5,size=12,ask=0)  # Plot to screen
                 newplot = None
               ymargin = getpct(compress(sspec,nspec),5), getpct(compress(sspec,nspec),95)
               ymargin = add(ymargin,multiply([-0.5,0.5],diff(ymargin)[0]))
               PG.newbox(add([-30,30],extrema(wavelengths)),ymargin,ch=2)
               PG.xlabel("\gl",ch=2)
               PG.ylabel("I",ch=2)
               PG.drawpt(wavelengths,nspec,ps=1,ch=1)
               PG.drawln(wavelengths,sspec,lw=1,ls=1,ci=4)
               PG.drawln(wavelengths,fitted,lw=2,ls=2,ci=2)
              
               ymargin = getpct(compress(sspec,residuals),5), getpct(compress(sspec,residuals),95)
               ymargin = add(ymargin,multiply([-0.5,0.5],diff(ymargin)[0]))
               PG.newbox(add([-30,30],extrema(wavelengths)),ymargin,ch=2)
               PG.xlabel("\gl",ch=2)
               PG.ylabel("Residuals",ch=2)
               PG.drawpt(wavelengths,residuals,ps=1,ch=1)
               PG.drawln(wavelengths,sspec,lw=1,ls=1,ci=4)
   	   if len(uorders)>1 and interactive: raw_input("<< cr >> ")
       PG.closedev(ask=0)
    absorb = divz(nspec,fitted)
    absorb = where(atmos,absorb,1.0)
    fitted = fitted / median(fitted)
    return fitted, absorb

#def flux(fitsfile,slitnum1,slitnum2,order,interactive,plotcc,smffile,lagtest,ffitsfile,maskreg,bands):
def flux(fitsfile,interactive,plotcc,smffile,lagtest,ffitsfile,maskreg,bands):
    L = [l.split() for l in open(smffile,"r").readlines() if l.split()[0] in ["HOLE","SLIT"]]
    slitids = [l[1].strip() for l in L]
    slitwidths = [float(l[6]) for l in L]

    if ffitsfile:
       ufitsfiles = [ffitsfile,fitsfile]
    else:
       ufitsfiles = [fitsfile]

    for ufitsfile in ufitsfiles:
        dirname = os.path.dirname(ufitsfile)
        if not dirname: dirname = "."
        params = open("%s/parameters.dat" % (dirname),"r")
        plines = [line for line in params if line[0] != "#"]
        params.close()

        slitnum1,slitnum2 = [int(line.split()[2]) for line in plines if "slitnum" in line]
        order = [int(line.split()[2]) for line in plines if "ordernum" in line][0]

        f = FITS(ufitsfile)

        if ufitsfile is fitsfile:
           ids = [f["obj%03d" % (nob+1)].strip() for nob in range(f["naxis2"])]
           widths = array([slitwidths[slitids.index(id1)] for id1 in ids])

        data = f.data()

        crval1 = f["crval1"]
        cdelt1 = f["cdelt1"]
        crpix1 = f["crpix1"]
        dcflag = f["dc-flag"]
        naxis1 = f["naxis1"]
        naxis3 = f["naxis3"]

        wavelengths = (arange(naxis1)+1.0-crpix1)*cdelt1 + crval1
        if dcflag == 1: wavelengths = power(10,wavelengths)

    # Least Squares Fit
    #A = polynom(wavelengths,order)
    #print A.shape   # A.shape --> (o+1,n), where n = len(wavelengths)
    #coefficients = linear_least_squares(transpose(A),nspec)[0]
    #print coefficients
    # thus nspec can be approximated, in a "least squares" sense by
    # nspec \approx dot(coefficients,A)
    #residuals = nspec - dot(coefficients,A)
    #rms_scatter = sqrt(mean(power(residuals,2)))
    #print rms_scatter

        Bb1,Bb2,Ab1,Ab2,Wb1,Wb2,Vb1,Vb2 = bands
        Bband = between(wavelengths,Bb1,Bb2)
        Aband = between(wavelengths,Ab1,Ab2)
        WaterBand = between(wavelengths,Wb1,Wb2)
        VeryRedband = between(wavelengths,Vb1,Vb2)

    #Bband = between(wavelengths,6850,6960)
    #Aband = between(wavelengths,7570,7700)
    #WaterBand = between(wavelengths,7140,7360)
    #VeryRedband = between(wavelengths,9270,9770)
        atmos = Bband + WaterBand + Aband + VeryRedband
        if maskreg:
            for reg in maskreg:
                print reg
                atmos = atmos + between(wavelengths,reg[0],reg[1])

        if ufitsfile is ufitsfiles[0]:
           spec = data[0][slitnum2]
           noise = data[1][slitnum2]
           flat = data[2][slitnum2]
           fitted = fitshape(wavelengths,spec,noise,flat,0*atmos,order,interactive)[0]
           uwaves = wavelengths

        if ufitsfile is fitsfile:
           fitted = splev(wavelengths,splrep(uwaves,fitted,s=0,k=3))
           fitted = fitted / median(fitted)
           spec = data[0][slitnum1]
           noise = data[1][slitnum1]
           flat = data[2][slitnum1]
           absorb = fitshape(wavelengths,spec,noise,flat,atmos,order,interactive)[1]
           fwidth = slitwidths[slitnum1]
    

#    if 0:
#        # Input fitted, absorb, fwidth
#        if wavelengths[-1] > 7600 and wavelengths[0] < 7600: fitted,absorb,fwidth,amp = pickle.load(open(bin))
#        else: fitted,absorb,fwidth = pickle.load(open(bin))
#        bin1f = FITS(bin.replace("_flux.bin",".fits"))
#        bin1crval = bin1f["crval1"]
#        bin1cdelt = bin1f["cdelt1"]
#        bin1crpix = bin1f["crpix1"]
#        bin1dcflag = bin1f["dc-flag"]
#        bin1f.close()
#        bin1waves = (arange(fitted.shape[0])+1-bin1crpix)*bin1cdelt + bin1crval
#        if bin1dcflag: bin1waves = power(10,bin1waves)
#        bin1fns = ["fitted","absorb"]
#        print extrema(wavelengths)
#        print extrema(bin1waves)
#        print amp,fwidth
##        exit()
#        for bin1fn in bin1fns:
#           exec("%s = splev(wavelengths,splrep(bin1waves,%s))" % (bin1fn,bin1fn))
#        fitted = fitted/median(fitted)

    if naxis3 is 3:
        spec, noise, flat = data
    elif naxis3 is 4:
        spec, noise, flat, lamp = data
    else:
        raise "not a valid naxis3 size"
    flat = flat / widths[::,NewAxis]

    # Output fitted, absorb, fwidth, amp 
#    t = open(fitsfile.replace("_2p_1spec.fits","_1spec_flux.bin"),"wb")
    if wavelengths[-1] > 7600 and wavelengths[0] < 7600:
       # Convolution
       if plotcc: PG.opendev("/xs",nx=2,ny=3,land=0,asp=0.75,size=12,ask=0)
       lags,fluxes,amps = transpose([Aconvolve(slit,absorb,wavelengths,cdelt1,interactive,plotcc,k,slitnum1) for k,slit in enumerate(spec)])
       if plotcc: PG.closedev(); time.sleep(0.05)
       if not bin: amp = amps[slitnum1]
       amps = amps / amp
       new_lags = lag_smf(ids,lags,fluxes,amps,smffile,lagtest)
       deltas = arange(-2,2.1,1.0,Float32)
       amps = asarray([GetAmp(slit,absorb,wavelengths,cdelt1,new_lags[k],deltas=deltas) for k,slit in enumerate(spec)])
       absorb_all = asarray([ sum(amps[j][::,NewAxis]*asarray([VTKImageShift(absorb,l1,0,numret=1) for l1 in add(l,deltas)])) for j,l in enumerate(new_lags)])
       fitted_all = asarray([VTKImageShift(fitted,l,0,numret=1) for l in new_lags])
       fluxing_all = absorb_all * fitted_all
#       pickle.dump((fitted,absorb,fwidth,amp),t)
    else:
       fluxing_all = (absorb * fitted)[NewAxis,::]
#       pickle.dump((fitted,absorb,fwidth),t)
#    t.close()


    # Flux spectra
    normflat = divz(flat,median(transpose(flat))[::,NewAxis])
    spec2 = divz(spec,fluxing_all*normflat)
    noise2 = divz(noise,fluxing_all*normflat)
    output = concatenate([data,[spec2,noise2]],0)
    naxes = array(output.shape)[::-1].tolist()

    # Write fits file
    fits = fitsfile.replace("_2p_1spec.fits","_1spec.fits")
    cf=FITS(fits,readonly=0,create=1)
    cf.copyheader(f)
    cf.resize(naxes)
    cf.putdata(output)
    cf.close()
    f.close()
    print fits

bin = None
interactive = False
fitsfile = None
ffitsfile = None
slitnum1 = None
slitnum2 = None
smffile = None
order = None
plotcc = False
lagtest = False
maskreg = []
Bb1,Bb2 = 6850,6960
Ab1,Ab2 = 7570,7700
Wb1,Wb2 = 7140,7360
Vb1,Vb2 = 9270,9770

n = 0 
for arg in sys.argv:
    #if arg in ("-s1","--slitnum1"): slitnum1 = int(sys.argv[n+1])
    #elif arg in ("-s2","--slitnum2"): slitnum2 = int(sys.argv[n+1])
    #elif arg in ("-bin"): bin = sys.argv[n+1]
    if arg in ("-f","--fitsfile"): fitsfile = sys.argv[n+1]
    elif arg in ("-ff","--ffitsfile"): ffitsfile = sys.argv[n+1]
    elif arg in ("-l","--lag"): lagtest = True
    elif arg in ("-m","--mask"): smffile = sys.argv[n+1]
    elif arg in ("-mr","--maskreg"): maskreg = eval(sys.argv[n+1])
    elif arg in ("-pcc","--plotcc"): plotcc = True
    #elif arg in ("-o","--order"): order = int(sys.argv[n+1])
    elif arg in ("-i","--interactive"): interactive = True
    elif arg in ("-Bb1","--Bband1"): Bb1 = int(sys.argv[n+1])
    elif arg in ("-Bb2","--Bband2"): Bb2 = int(sys.argv[n+1])
    elif arg in ("-Ab1","--Aband1"): Ab1 = int(sys.argv[n+1])
    elif arg in ("-Ab2","--Aband2"): Ab2 = int(sys.argv[n+1])
    elif arg in ("-Wb1","--Waterband1"): Wb1 = int(sys.argv[n+1])
    elif arg in ("-Wb2","--Waterband2"): Wb2 = int(sys.argv[n+1])
    elif arg in ("-Vb1","--VRedband1"): Vb1 = int(sys.argv[n+1])
    elif arg in ("-Vb2","--VRedband2"): Vb2 = int(sys.argv[n+1])
    elif arg in ("-h","--help"):
        help()
        os._exit(1)
    n += 1

bands = Bb1,Bb2,Ab1,Ab2,Wb1,Wb2,Vb1,Vb2
print fitsfile,smffile#,slitnum1,slitnum2,order 

if (fitsfile and smffile):
    #flux(fitsfile,slitnum1,slitnum2,order,interactive,plotcc,smffile,lagtest,ffitsfile,maskreg,bands)
    flux(fitsfile,interactive,plotcc,smffile,lagtest,ffitsfile,maskreg,bands)
else:
    help()
    sys.exit()
