#!/usr/bin/env python
#
# Generic functions used by ReportLab and Cheetah, returns dictionaries and variables.
#
# Last modified 7-1-05
#
# TODO:
# Create generic tools for both ReportLab and Cheetah.

import glob,getpass,os,sys
from FITS import *

### Headers
def headreadN(objdat,input):
### headreadN((prefix,file,adjust),input)
### Reads header from a list of files. Takes a single input value in the form of a dictionary in
### order to determine which value to read from. There is an adjust factor in case the read value
### is offset by some amount.
    prefix,file,adjust = objdat
    inval = input.values()
    list = glob.glob(file)
    dict = {}
    for fits in list:
        fitsfile = open(fits,"rb")
        obj = "%s%i" % (prefix,inval[0]-adjust)
        headvalue = None 
        while 1:
            headstring = fitsfile.read(80)
            if obj in headstring:
                headvalue = float(headstring.split()[2])
            if headvalue is not None:
                break
        dict[fits]=headvalue

    print "HeadreadN %s - done" % (obj)
    return dict

def headreadS(objdat):
### Reads header from a single file glob. Returns a dictionary, a file name matched with a value.
    obj,file = objdat
    dict = {}
    fits = (glob.glob(file))[0]
    fitsfile = open(fits,"rb")
    headvalue = None 
    while 1:
        headstring = fitsfile.read(80)
        if obj in headstring:
            headvalue = float(headstring.split()[2])
        if headvalue is not None:
            break
    dict[fits]=headvalue

    print "Headread %s - done" % (obj)
    return dict

def headread(objdat):
### Reads header from a list of files. Returns a dictionary, a file name matched with a value.
    obj,file = objdat
    list = glob.glob(file)
    dict = {}
    for fits in list:
        fitsfile = open(fits,"rb")
        headvalue = None 
        while 1:
            headstring = fitsfile.read(80)
            if obj in headstring:
                headvalue = float(headstring.split()[2])
            if headvalue is not None:
                break
        dict[fits]=headvalue

    print "Headread %s - done" % (obj)
    return dict

### Header Target
def target():
    try:
        pipeline = glob.glob("*.pipeline")[0]
    except IndexError:
        print "*.pipeline file not found"
        sys.exit()  
    
    tar = pipeline.replace(".pipeline","")    
    print "Target %s - done" % (tar)
    return tar

#def exptime():
#    dict = headread(["EXPTIME","*spec[0-9][0-9][0-9][0-9].fits"])
#    exp = 0
#    valuelist = dict.values()
#    for i in range(len(valuelist)):
#        exp = exp + valuelist[i]
#    return exp

def exptime():
    L = glob.glob("*_multi.fits")
    E = [FITS(l)["EXPTIME"] for l in L]
    return E

def check(line,condition,correction):

    test = os.path.isfile(condition[:-1])
    if correction is not "":
        line = correction
    if test is True:
        line = "%s: OK" % (line[:-2])
        print "Check %s - done" % (condition[:-1])
    elif test is False:
        line = "%s: FAIL" % (line[:-2])
        print "Check %s - incomplete" % (condition[:-1])
    return line,test

def user():
    user = getpass.getuser()
    return user

def frames():
    science = glob.glob("*_spec[0-9][0-9][0-9][0-9].fits")
    thar = glob.glob("*_lamp[0-9][0-9][0-9][0-9].fits")
    #quartz = glob.glob("*_slit[0-9][0-9][0-9][0-9].fits")
    #return science,thar,quartz
    return science,thar
