#!/usr/bin/env python
#
# Program used to generate index.html for all targets, used to simplify directory navigation
#
# Last modified 12-14-04


from Cheetah.Template import Template
import glob,os,time,sys

def plonecheetahvar(namespace,runid,dirlist):
    try: 
        os.remove("index.html")
        print "previous index.html found"
        print "removing index.html"
    except: print "no index.html found"
    indent = " "*6
    templateDef = """
    <HTML>
    <HEAD><TITLE>$title</TITLE></HEAD>
    <BODY>
     
      $info 
      $mikesetup
      $tarlink
      $dblink
      $datatable

    </BODY>
    </HTML>"""
    t = Template(templateDef,searchList=[namespace])

    t.title = runid
    umikesetup_commands = []
    udb = []
    uindexlist = []
    if len(dirlist) is not 0:
        for dir in dirlist:
            mikesetup_commands = [line.strip() for g in glob.glob("%s/*" % (dir)) if not os.path.isdir(g) for line in open(g,"r") if "mikesetup" in line]
            umikesetup_commands = umikesetup_commands + mikesetup_commands  
            db = glob.glob("%s/*.db" % (dir))
            udb = udb + db
            indexlist = glob.glob("%s/*/index.html" % (dir))
            uindexlist = uindexlist + indexlist
            #print uindexlist
    else:
        umikesetup_commands = [line.strip() for g in glob.glob("*") if not os.path.isdir(g) for line in open(g,"r") if "mikesetup" in line]
        udb = glob.glob("*.db")
        uindexlist = glob.glob("*/index.html")

    #print umikesetup_commands
    uindexlist.sort()
    #print uindexlist

    t.info = "<P>Each of the *multi.fits files are in the form of a data cube. The individual bands of the data cube are the following:<br>&nbsp;&nbsp;Band 1 - Sky<br>&nbsp;&nbsp;Band 2 - Object<br>&nbsp;&nbsp;Band 3 - Noise<br>&nbsp;&nbsp;Band 4 - Signal-to-Noise<br>&nbsp;&nbsp;Band 5 - Lamp</P>"
    t.mikesetup = "<P>Commands used to setup the pipeline:"
    for command in umikesetup_commands: t.mikesetup += "<BR>%s" % (command)
    t.mikesetup += "</P>"

    gztar = glob.glob("*.tar.gz")[0]
    t.tarlink = "<P>Archived file of all multispec fits files:<BR><a href='%s'>%s</a></P>\n" % (gztar,gztar)

    t.dblink = "<P>Database file used for reduction:"
    for db in udb: t.dblink += "<BR><a href='%s/pipeline_view'>%s</a>" % (db,os.path.basename(db))
    t.dblink += "</P>\n"


    tablebegin = "<P><FONT SIZE=2><TABLE BORDER=1 WIDTH=100% HEIGHT=100% ID='sortable' CLASS='listing'>\n"
    if len(dirlist) is not 0:
        tablehead = indent + "<THEAD>\n" + indent + "<TR><TH>Target<TH>Side<TH>Subdir<TH>Created<TH>Multispec<TH>Short Report<TH>Long Report<TH>Pipeline\n" + indent + "</THEAD>\n"
    else:
        tablehead = indent + "<THEAD>\n" + indent + "<TR><TH>Target<TH>Side<TH>Created<TH>Multispec<TH>Short Report<TH>Long Report<TH>Pipeline\n" + indent + "</THEAD>\n"

    tabletail = indent + "</TABLE></FONT></P>"
    tablebody = ""
#    tablebodyblue = ""
#    tablebodyred = ""

    n = 0 
    for folderpath in uindexlist:
        folder = os.path.dirname(folderpath)
        #print folder
        multilist = glob.glob("%s/*_multi.fits" % (folder))[0]
        shortlist = glob.glob("%s/*reportShort.pdf" % (folder))[0]
        longlist = glob.glob("%s/*reportLong.pdf" % (folder))[0]
        pipelist = glob.glob("%s/*pipeline" % (folder))[0]
        create = time.strftime("%b-%d-%Y %H:%M:%S", time.localtime(os.path.getctime(multilist)))
        pipe = os.path.basename(pipelist)

        color = ""
        targ = ""

        if "blue" in folder:
            color = "blue"
            targ = os.path.basename(folder).replace("blue","")

        elif "red" in folder:
            color = "red"
            targ = os.path.basename(folder).replace("red","")

        if len(dirlist) is not 0:
            subfolder = os.path.dirname(folder)
            tablebody = tablebody + indent + "<TR><TD>%s<TD>%s<TD>%s<TD>%s<TD align='center'><a href='%s'>%s</a><TD align='center'><a href='%s/index.html'>%s</a> <a href='%s'>%s</a><TD align='center'><a href='%s'>%s</a><TD><a href='%s/pipeline_view'>%s</a>\n" % (targ,color,subfolder,create,multilist.replace("+","_"),"fits",folder.replace("+","_"),"html",shortlist.replace("+","_"),"pdf",longlist.replace("+","_"),"pdf",pipelist.replace("+","_"),pipe)
        else:
            tablebody = tablebody + indent + "<TR><TD>%s<TD>%s<TD>%s<TD align='center'><a href='%s'>%s</a><TD align='center'><a href='%s/index.html'>%s</a> <a href='%s'>%s</a><TD align='center'><a href='%s'>%s</a><TD><a href='%s/pipeline_view'>%s</a>\n" % (targ,color,create,multilist.replace("+","_"),"fits",folder.replace("+","_"),"html",shortlist.replace("+","_"),"pdf",longlist.replace("+","_"),"pdf",pipelist.replace("+","_"),pipe)

#        if "blue" in folder:
#            color = "blue"
#            targ = os.path.basename(folder).replace("blue","")
#            if len(dirlist) is not 0:
#                subfolder = os.path.dirname(folder)
#                tablebodyblue = tablebodyblue + indent + "<TR><TD>%s<TD>%s<TD>%s<TD>%s<TD align='center'><a href='%s'>%s</a><TD align='center'><a href='%s/index.html'>%s</a> <a href='%s'>%s</a><TD align='center'><a href='%s'>%s</a><TD><a href='%s/pipeline_view'>%s</a>\n" % (targ,color,subfolder,create,multilist,"fits",folder,"html",shortlist,"pdf",longlist,"pdf",pipelist,pipe)
#            else:
#                tablebodyblue = tablebodyblue + indent + "<TR><TD>%s<TD>%s<TD>%s<TD align='center'><a href='%s'>%s</a><TD align='center'><a href='%s/index.html'>%s</a> <a href='%s'>%s</a><TD align='center'><a href='%s'>%s</a><TD><a href='%s/pipeline_view'>%s</a>\n" % (targ,color,create,multilist,"fits",folder,"html",shortlist,"pdf",longlist,"pdf",pipelist,pipe)
#        elif "red" in folder:
#            color = "red"
#            targ = os.path.basename(folder).replace("red","")
#            if len(dirlist) is not 0:
#                subfolder = os.path.dirname(folder)
#                tablebodyred = tablebodyred + indent + "<TR><TD>%s<TD>%s<TD>%s<TD>%s<TD align='center'><a href='%s'>%s</a><TD align='center'><a href='%s/index.html'>%s</a> <a href='%s'>%s</a><TD align='center'><a href='%s'>%s</a><TD><a href='%s/pipeline_view'>%s</a>\n" % (targ,color,subfolder,create,multilist,"fits",folder,"html",shortlist,"pdf",longlist,"pdf",pipelist,pipe)
#            else:
#                tablebodyred = tablebodyred + indent + "<TR><TD>%s<TD>%s<TD>%s<TD align='center'><a href='%s'>%s</a><TD align='center'><a href='%s/index.html'>%s</a> <a href='%s'>%s</a><TD align='center'><a href='%s'>%s</a><TD><a href='%s/pipeline_view'>%s</a>\n" % (targ,color,create,multilist,"fits",folder,"html",shortlist,"pdf",longlist,"pdf",pipelist,pipe)
#
        n += 1
    if tablebody is "":
        t.datatable = ""
    else:
        t.datatable = tablebegin + tablehead + tablebody + tabletail
    
#    if tablebodyblue is "":
#        t.datatableblue = ""
#    else:
#        t.datatableblue = tablebegin + tablehead + tablebodyblue + tabletail
#    
#    if tablebodyred is "":
#        t.datatablered = ""
#    else:
#        t.datatablered = tablebegin + tablehead + tablebodyred + tabletail

    content = t.respond()
    #print t 

    f = open("index.html","w")
    f.write(content)
    f.close()
    print "index.html complete"

def goplonecheetah(runid,dirlist):
    global cheetahdict
    cheetahdict = {}
    plonecheetahvar(cheetahdict,runid,dirlist)

#goplonecheetah()
